/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *                 Stephen Clavering <mozilla@clav.me.uk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function prefbarClearCookies() {
  var cookieman = Components.classes["@mozilla.org/cookiemanager;1"]
                            .getService(Components.interfaces.nsICookieManager);
  cookieman.removeAll();
}


function prefbarClearHistory() {
  var history = Components.classes["@mozilla.org/browser/global-history;2"]
                          .getService(Components.interfaces.nsIBrowserHistory);
  history.removeAllPages();

  // for firefox.  used to clear the history on back/forward buttons, and the
  // <browser/> session history
  var os = Components.classes["@mozilla.org/observer-service;1"]
                     .getService(Components.interfaces.nsIObserverService);
  os.notifyObservers(null, "browser:purge-session-history", "");
}


function prefbarClearLocationBar() {
  goPrefBar.ClearPref("general.open_location.last_url");

  // SeaMonkey 2.x
  var file = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("ProfD", Components.interfaces.nsIFile);
  file.append("urlbarhistory.sqlite");
  if (file.exists())
    file.remove(false);
}


function prefbarClearCache(aType) {
  var cacheService = Components.classes["@mozilla.org/network/cache-service;1"]
    .getService(Components.interfaces.nsICacheService);
  cacheService.evictEntries(aType);
}

function prefbarClearMemCache() {
  prefbarClearCache(Components.interfaces.nsICache.STORE_IN_MEMORY);
}

function prefbarClearDiskCache() {
  prefbarClearCache(Components.interfaces.nsICache.STORE_ON_DISK);
}

function prefbarClearAllCache() {
  prefbarClearMemCache();
  prefbarClearDiskCache();
}

function prefbarZoomReduce() {
  if ("FullZoom" in window)
    FullZoom.reduce();
  else
    ZoomManager.prototype.getInstance().reduce();
}
function prefbarZoomEnlarge() {
  if ("FullZoom" in window)
    FullZoom.enlarge();
  else
    ZoomManager.prototype.getInstance().enlarge();
}

//
// "Kill Flash" button
//

function prefbarKillFlash() {
  var frames = prefbarGetFrames(window._content.window);
  for (var frameindex in frames) {
    var page = frames[frameindex].document;

    // "Twice-Cooked Method" and <embed>

    var embeds = page.getElementsByTagName("embed");

    for(var i = embeds.length - 1; i >= 0; i--) {
      var current = embeds[i];

      if(current.type =="application/x-shockwave-flash") {
        if(current.parentNode.nodeName.toLowerCase() == "object")
          current = current.parentNode;
        prefbarRemoveElement(page, current);
      }
    }

    // "Satay Method" and <object>

    var objects = page.getElementsByTagName("object");

    for(var i = objects.length - 1; i >= 0; i--) {
      var current = objects[i];

      if(current.type =="application/x-shockwave-flash")
        prefbarRemoveElement(page, current);
    }
  }
}

function prefbarGetFrames(owindow) {
  var winarray = new Array();
  winarray.push(owindow);
  var len = owindow.frames.length;
  for (var index = 0; index < len; index++)
    winarray = winarray.concat(prefbarGetFrames(owindow.frames[index]));
  return winarray;
}

function prefbarRemoveElement(page, element) {
  var width = element.width;
  var height = element.height;
  if (width && height) {
    var div = page.createElement("DIV");
    var text = page.createTextNode(" ");
    div.appendChild(text);

    element.parentNode.replaceChild(div, element);

    div.setAttribute("style", "height: " + height + "px; width: " + width + "px; border: 1px solid black;");
  }
  else
    element.parentNode.removeChild(element);
}

//
// Proxy serverlist
//

function prefbarSetProxy(str) {
  if (str == "")
    goPrefBar.SetPref("network.proxy.type", 0);
  else {
    var args = str.split(":");
    var host;
    var port;

    if (args.length == 3 && args[0] == "socks") {
      host = args[1];
      port = Number(args[2]);

      goPrefBar.SetPref("network.proxy.socks", host);
      goPrefBar.SetPref("network.proxy.socks_port", port);
      goPrefBar.ClearPref("network.proxy.http");
      goPrefBar.ClearPref("network.proxy.http_port");
      goPrefBar.ClearPref("network.proxy.ssl");
      goPrefBar.ClearPref("network.proxy.ssl_port");
      goPrefBar.ClearPref("network.proxy.ftp");
      goPrefBar.ClearPref("network.proxy.ftp_port");
      goPrefBar.ClearPref("network.proxy.gopher");
      goPrefBar.ClearPref("network.proxy.gopher_port");
    }
    else if (args.length == 2) {
      host = args[0];
      port = Number(args[1]);

      goPrefBar.SetPref("network.proxy.http", host);
      goPrefBar.SetPref("network.proxy.http_port", port);
      goPrefBar.SetPref("network.proxy.ssl", host);
      goPrefBar.SetPref("network.proxy.ssl_port", port);
      goPrefBar.SetPref("network.proxy.ftp", host);
      goPrefBar.SetPref("network.proxy.ftp_port", port);
      goPrefBar.SetPref("network.proxy.gopher", host);
      goPrefBar.SetPref("network.proxy.gopher_port", port);
      goPrefBar.ClearPref("network.proxy.socks");
      goPrefBar.ClearPref("network.proxy.socks_port");
    }
    else {
	    goPrefBar.msgAlert(window, 'Wrong proxy setting: "' + str + '"');
	    return;
    }

    // Maybe we shouldn't set the Proxy Type to "Manual" automatically,
    // since some users may want to use "Proxy Serverlist" *and* "Proxy
    // Menulist" at once...
    goPrefBar.SetPref("network.proxy.type", 1);
  }
}

function prefbarGetProxy() {
  if (goPrefBar.GetPref("network.proxy.type") != 1)
    return "";

  if (goPrefBar.GetPref("network.proxy.socks") != "") {
    return "socks:" +
           goPrefBar.GetPref("network.proxy.socks") + ":" +
           goPrefBar.GetPref("network.proxy.socks_port");
  }

  return goPrefBar.GetPref("network.proxy.http") + ":" +
         goPrefBar.GetPref("network.proxy.http_port");
}

//
// "Pipelining" checkbox
//

function prefbarSetPipelining(value) {
  goPrefBar.SetPref("network.http.proxy.pipelining", value);
  goPrefBar.SetPref("network.http.pipelining", value);
}

function prefbarGetPipelining() {
  return (goPrefBar.GetPref("network.http.proxy.pipelining") &&
          goPrefBar.GetPref("network.http.pipelining"));
}

//
// "Resize" menulist
//

function prefbarSetResolution(value) {
  var valueArray = value.split("x");
  window.outerWidth = valueArray[0];
  window.outerHeight = valueArray[1];
}

function prefbarGetResolution() {
  return window.outerWidth + "x" + window.outerHeight;
}

//
// "Save Page" button
//

var prefbarSPFilename = null;
var prefbarSPOldGetDefaultFileName = null;
var prefbarSPGetDefaultFileName = function(param1, param2, param3, param4) {
  if (prefbarSPFilename) {
    var retval = prefbarSPFilename;
    prefbarSPFilename = null;
    return retval;
  }
  else
    return prefbarSPOldGetDefaultFileName(param1, param2, param3, param4);
}

function prefbarSavePage() {
  var title = window._content.document.title;
  var uri = window._content.location;
  var doc = window._content.document;
  var rchar = "_";

  title = title.replace(/\/|\\|:|\*|\?|\"|<|>|\|/g, rchar);
  title = title.replace(/\s/g, rchar);
  if (title.match(new RegExp("^" + rchar + "+$"))) title = "";

  // Filename is the modified title
  prefbarSPFilename = title;

  if (!prefbarSPOldGetDefaultFileName) {
    if ("getDefaultFileName" in this) {
	    prefbarSPOldGetDefaultFileName = this.getDefaultFileName;
	    this.getDefaultFileName = prefbarSPGetDefaultFileName;
    }
    else {
	    goPrefBar.msgAlert(window, "Save Page feature not available for the mozilla version you use!");
	    return;
    }
  }

  // Save function used by Mozilla since 1.8 and SeaMonkey
  if ("internalSave" in this) {
    // We want to use cached data because the document is currently visible.
    var dispHeader = null;
    try {
      dispHeader =
        doc.defaultView
        .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
        .getInterface(Components.interfaces.nsIDOMWindowUtils)
        .getDocumentMetadata("content-disposition");
    } catch (ex) {
      // Failure to get a content-disposition is ok
    }

    internalSave(uri, doc, null, dispHeader,
                 doc.contentType, false, null, null);
  }
  else
    goPrefBar.msgAlert(window, "Save Page feature not available for the mozilla version you use!");
}

//
// external application support
//

function prefbarExecute(path, arguments) {
  if (arguments == undefined) arguments = new Array;

  var executable = Components.classes["@mozilla.org/file/local;1"]
    .createInstance(Components.interfaces.nsILocalFile);
  executable.initWithPath(path);

  if (!executable.exists()) {
    goPrefBar.msgAlert(window, "Error - executable '" + path + "' not found");
    return;
  }

  try {
    var process = Components.classes["@mozilla.org/process/util;1"]
                            .createInstance(Components.interfaces.nsIProcess);

    process.init(executable);

    var pid = new Object();
    process.run(false, arguments, arguments.length, pid);
  }
  catch (ex) {
    goPrefBar.msgAlert(window, "Error - couldn't execute '" + path + "'");
  }
}

//
// Useragent Menulist
// This code handles useragent, appname, appversion and platform spoofing
// If more than useragent is needed, then prefix the value with "js:"
// and use Javascript syntax to set the variables "useragent", "appname",
// "appversion" and "platform"
//

function prefbarSetUseragent(value) {
  // This are the variables that may be changed in the short Javascript
  // in menuitems value
  var useragent;
  var appname;
  var appversion;
  var platform;

  if (value.substr(0,3) == "js:")
    eval(value.substr(3));
  else {
    // XXX Hack to get backwards compatibility
    // In new PrefBar versions the default item in an list is marked by the
    // value "PREFBARDEFAULT". This allows to have the default item
    // everywhere in the list and doesn't confuse the users (see
    // PrefBar-Bug 5733) In old versions simply the first item was the
    // default. As we don't update the items of the "User Agent Menulist",
    // we have to be backwards compatible for this one item.
    if (value == "") value = undefined;

    // If value is PREFBARDEFAULT then we have to reset the preference
    if (value == "PREFBARDEFAULT") value = undefined;

    useragent = value;
  }

  // Either set or reset the variables that may be changed using the user
  // agent menulist.
  var pref = "general.useragent.override";
  if (useragent == undefined)
    goPrefBar.ClearPref(pref);
  else
    goPrefBar.SetPref(pref, useragent);
  pref = "general.appname.override";
  if (appname == undefined)
    goPrefBar.ClearPref(pref);
  else
    goPrefBar.SetPref(pref, appname);
  pref = "general.appversion.override";
  if (appversion == undefined)
    goPrefBar.ClearPref(pref);
  else
    goPrefBar.SetPref(pref, appversion);
  pref = "general.platform.override";
  if (platform == undefined)
    goPrefBar.ClearPref(pref);
  else
    goPrefBar.SetPref(pref, platform);
}

function prefbarGetUseragent(context) {
  var prefBranch = goPrefBar.PrefBranch;
  var defaultset = !prefBranch.prefHasUserValue("general.useragent.override");

  // XXX Hack to get backwards compatibility
  // In new PrefBar versions the default item in an list is marked by the
  // value "PREFBARDEFAULT". This allows to have the default item
  // everywhere in the list and doesn't confuse the users (see
  // PrefBar-Bug 5733) In old versions simply the first item was the
  // default. As we don't update the items of the "User Agent Menulist",
  // we have to be backwards compatible for this one item.
  if (defaultset) {
    var defaultexists = false;
    var len = context.items.length;
    for (var index = 0; index < len; index++) {
      if (context.items[index][1] == "PREFBARDEFAULT") {
        defaultexists = true;
        break;
      }
    }

    if (defaultexists)
      context.value = "PREFBARDEFAULT";
    else
      context.value = context.items[0][1];

    return;
  }

  var uavalue = goPrefBar.GetPref("general.useragent.override");
  var len = context.items.length;
  for (var index = 0; index < len; index++) {
    var curval = context.items[index][1];
    if (curval.substr(0,3) == "js:") {
      var useragent;
      var appname;
      var appversion;
      var platform;
      eval(curval.substr(3));
      if (useragent == uavalue) {
        context.value = curval;
        break;
      }
    }
    else {
      if (curval == uavalue) {
        context.value = curval;
        break;
      }
    }
  }
}

//
// Languages Menulist
// "Auto-Configures" using the settings done in the content languages GUI
// of the browser!
//

function prefbarSetLanguage(value) {
  var prefstr = goPrefBar.PrefBranch.getComplexValue("intl.accept_languages", Components.interfaces.nsIPrefLocalizedString).data;
  var langs = prefstr.split(/\s*,\s*/);
  var newlangs = Array(value);

  var len = langs.length;
  for (var index = 0; index < len; index++) {
    if (langs[index] != value)
      newlangs.push(langs[index]);
  }

  prefstr = newlangs.join(", ");
  goPrefBar.SetPref("intl.accept_languages", prefstr);
}

function prefbarGetLanguage(context) {
  var prefstr = goPrefBar.PrefBranch.getComplexValue("intl.accept_languages", Components.interfaces.nsIPrefLocalizedString).data;
  var langs = prefstr.split(/\s*,\s*/);

  context.items = Array();

  var len = langs.length;
  for (var index = 0; index < len; index ++) {
    context.items.push(Array(langs[index], langs[index]));
  }

  context.value = langs[0];
}

//
// Restore Tab Button
//

//var prefbarRestoreTabClosed = new Array();
//var prefbarRestoreTabRemoveTab = function(aTab) {
//  var max_cache = goPrefBar.GetPref("extensions.prefbar.btnrestoretab.maxcache");
//
//    if (aTab.localName != "tab")
//        aTab = gBrowser.mCurrentTab;
//
//    // Older browsers don't have "linkedBrowser".
//    if (!("linkedBrowser" in aTab)) {
//        gBrowser.prefbarOldRemoveTab(aTab);
//        return;
//    }
//
//    var browser = aTab.linkedBrowser;
//    var position = null;
//    // SeaMonkey
//    if ("getTabIndex" in gBrowser) position = gBrowser.getTabIndex(aTab);
//    // Firefox 1.5
//    if ("_tPos" in aTab) position = aTab._tPos;
//
//    prefbarRestoreTabClosed.push(Array(position, browser.sessionHistory));
//    if (prefbarRestoreTabClosed.length > max_cache)
//        prefbarRestoreTabClosed.shift();
//    gBrowser.prefbarOldRemoveTab(aTab);
//}

//function prefbarRestoreTabInit() {
//    // Firefox 2.0 brings it's own restore tab backend.
//    // If this exists, then exit now! We will call the internal function
//    // to restore tabs, so our backend is now exclusively for SeaMonkey!
//    if ("undoCloseTab" in this) {
//        goPrefBar.dump("prefbarRestoreTab: Cancelled init. Firefox backend will be used!");
//        return;
//    }
//
//    // For some reason, sometimes gBrowser isn't defined
//    if (!gBrowser) gBrowser = document.getElementById("content");
//
//    // MultiZilla brings its own restore tab backend.
//    // If this exists, then exit now! We will call the MultiZilla function
//    // to restore tabs.
//    if ("restoreTab" in gBrowser) {
//        goPrefBar.dump("prefbarRestoreTab: Cancelled init. MultiZilla backend will be used!");
//        return;
//    }
//
//    if ("prefbarOldRemoveTab" in gBrowser) return;
//    if (window.tablib) tablib.init();
//    window.gBrowser.prefbarOldRemoveTab = gBrowser.removeTab;
//    window.gBrowser.removeTab = prefbarRestoreTabRemoveTab;
//}

function prefbarRestoreTab() {
    // Call Firefox 2.0 restore tab backend, if exists.
    if ("undoCloseTab" in this) {
        goPrefBar.dump("prefbarRestoreTab: Calling internal 'undoCloseTab'");
        undoCloseTab();
        return;
    }

    // Call MultiZilla restore tab backend, if exists.
    if ("restoreTab" in gBrowser) {
        goPrefBar.dump("prefbarRestoreTab: Calling MultiZilla 'gBrowser.restoreTab'");
        gBrowser.restoreTab(0);
        return;
    }

    // If we get up to here, then "Restore Tab" didn't work
    goPrefBar.msgAlert(window, "Restore Tab doesn't currently work in the browser, you use.\nPlease file a bug at http://prefbar.mozdev.org/");

//    if (!prefbarRestoreTabClosed.length) return;
//    var aTab = prefbarRestoreTabClosed.pop();
//    var newTab = gBrowser.addTab("about:blank");
//    if (gBrowser.moveTabTo) gBrowser.moveTabTo(newTab, aTab[0]);
//    gBrowser.selectedTab = newTab;
//
//    var oldHistory = aTab[1];
//    var newHistory = gBrowser.webNavigation.sessionHistory;
//    newHistory.QueryInterface(Components.interfaces.nsISHistoryInternal);
//
//    var len = oldHistory.count;
//    for (var i = 0; i < len; i++) {
//        newHistory.addEntry(oldHistory.getEntryAtIndex(i, false), true);
//    }
//
//    if (oldHistory.count) gBrowser.webNavigation.gotoIndex(oldHistory.index);
}

//
// Proxies checkbox
//

function prefbarGetProxyEnabled() {
  var type = goPrefBar.GetPref("network.proxy.type");
  return (type != 0);
}

function prefbarSetProxyEnabled(value) {
  var typepref = "network.proxy.type";
  var lasttypepref = "extensions.prefbar.btnproxycheck.lasttype";
  if (value == false) {
    var type = goPrefBar.GetPref(typepref);
    goPrefBar.SetPref(lasttypepref, type);
    goPrefBar.SetPref(typepref, 0);
  }
  else {
    var lasttype = goPrefBar.GetPref(lasttypepref, 1);
    if (lasttype == 0) lasttype = 1;
    goPrefBar.SetPref(typepref, lasttype);
  }
}

//
// Tab dependent checkboxes
//

// Javascript
function prefbarSetTabJavascript(value) {
  prefbarGetDocShell().allowJavascript = value;
}
function prefbarGetTabJavascript() {
  return prefbarGetDocShell().allowJavascript;
}
// Images
function prefbarSetTabImages(value) {
  prefbarGetDocShell().allowImages = value;
}
function prefbarGetTabImages() {
  return prefbarGetDocShell().allowImages;
}
// Plugins
function prefbarSetTabPlugins(value) {
  prefbarGetDocShell().allowPlugins = value;
}
function prefbarGetTabPlugins() {
  return prefbarGetDocShell().allowPlugins;
}
function prefbarGetDocShell() {
  var docShell = getBrowser().docShell;
  docShell.QueryInterface(Components.interfaces.nsIDocShell);
  return docShell;
}

//
// A "PrefBar-API" function, which reads a string from clipboard.
// Only for external buttons. Not used in PrefBar, so far.
//

function prefbarReadClipboard(useSelClipboard) {
  // Prepare nsITransferable
  var trans = Components.classes["@mozilla.org/widget/transferable;1"]
                        .createInstance(Components.interfaces.nsITransferable);
  trans.addDataFlavor("text/unicode");

  // Read clipboard data to nsITransferable
  var clip = Components.classes["@mozilla.org/widget/clipboard;1"]
                       .getService(Components.interfaces.nsIClipboard);
  var cptype = clip.kGlobalClipboard;
  if (useSelClipboard && clip.supportsSelectionClipboard())
    cptype = clip.kSelectionClipboard;
  clip.getData(trans, cptype);

  // Get data out of nsITransferable
  var str       = new Object();
  var strLength = new Object();

  try {
    trans.getTransferData("text/unicode", str, strLength);
  } catch(e) { return false; }

  if (!str) return false;

  str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
  str = str.data.substring(0, strLength.value / 2);

  // Return string with dropped spaces in front and at the end.
  return str.replace(/^\s*/, "").replace(/\s*$/, "");
}

//
// General stuff for toggling plugins in addon manager
//

// Used to get plugin object array from plugin manager
// May also be used to check, if Plugin manager exists
function prefbarGetPluginTags() {
  if (!("nsIPluginHost" in Components.interfaces)) return false;
  var phs = Components.classes["@mozilla.org/plugin/host;1"]
                      .getService(Components.interfaces.nsIPluginHost);
  var plugins = phs.getPluginTags({ });
  return plugins;
}
function prefbarSetPlugin(aRegEx, aValue, aName) {
  if (!aName) aName = aRegEx.toString().replace(/[^a-z ]/gi, "");

  var plugins = prefbarGetPluginTags();
  if (!plugins) return;
  var found = false;
  for (var i = 0; i < plugins.length; i++) {
    if (plugins[i].name.match(aRegEx)) {
      plugins[i].disabled = !aValue;
      found = true;
    }
  }

  if (!found) goPrefBar.msgAlert(window, "No " + aName + " plugin found!");
}
function prefbarGetPlugin(aRegEx) {
  var plugins = prefbarGetPluginTags();
  if (!plugins) return false;
  for (var i = 0; i < plugins.length; i++) {
    if (plugins[i].name.match(aRegEx) && !plugins[i].disabled) return true;
  }
  return false;
}

//
// Java checkbox
//

//Java(TM) Plug-in                  <-- Linux
//InnoTek OS/2 Kit for Java Plug-in <-- OS/2
//Java(TM) Platform SE 6 U16        <-- Windows
//Java Deployment Toolkit 6.0.160.1 <-- Don't match for this!
var prefbarRegExJava = /(^| )java.*(platform|plug-in)/i;
function prefbarSetJava(aValue) {
  // Set both (Pref and Plugin disabled status), if possible
  goPrefBar.SetPref("security.enable_java", aValue);
  prefbarSetPlugin(prefbarRegExJava, aValue, "Java");
}
function prefbarGetJava() {
  return prefbarGetPlugin(prefbarRegExJava);
}

//
// Flash checkbox
//

var prefbarRegExFlash = /^shockwave flash/i;
function prefbarSetFlash(aValue) {
  prefbarSetPlugin(prefbarRegExFlash, aValue, "Flash");
}
function prefbarGetFlash() {
  return prefbarGetPlugin(prefbarRegExFlash);
}
